<?php

declare(strict_types=1);

namespace mirzaev\calculator\controllers;

use mirzaev\calculator\controllers\core;
use mirzaev\calculator\models\calculators_model as calculators;

use Twig\Loader\FilesystemLoader;
use Twig\Environment as view;

/**
 * Контроллер основной страницы
 *
 * @package mirzaev\calculator\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class calculator_controller extends core
{
    /**
     * Калькулятор
     *
     * HTML-код с калькулятором
     *
     * @param array $vars Параметры
     */
    public function index(array $vars = []): ?string
    {
        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculator' . DIRECTORY_SEPARATOR . 'index.html', $vars);
    }

    /**
     * Модуль: "тип пользователя"
     *
     * HTML-код с кнопками: "физическое лицо" и "юридическое лицо"
     *
     * @param array $vars Параметры
     */
    public function buyer(array $vars = []): ?string
    {
        // Инициализация параметров
        $vars['buyer'] = $vars['value'] ?? 'individual';

        // Удаление параметров
        unset($vars['value']);

        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'buyer.html', $vars);
    }

    /**
     * Модуль: "сложность"
     *
     * HTML-код с кнопками: "легко", "средне" и "сложно"
     *
     * @param array $vars Параметры
     */
    public function complexity(array $vars = []): ?string
    {
        // Инициализация параметров
        $vars['complexity'] = $vars['value'] ?? 'medium';

        // Удаление параметров
        unset($vars['value']);

        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'complexity.html', $vars);
    }

    /**
     * Модуль: "меню"
     *
     * HTML-код с кнопками добавления калькуляторов
     *
     * @param array $vars Параметры
     */
    public function menu(array $vars = []): ?string
    {
        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'menu.html', $vars);
    }

    /**
     * Модуль: "результат"
     *
     * HTML-код с данными результата калькуляции
     *
     * @param array $vars Параметры
     */
    public function result(array $vars = []): ?string
    {
        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'result.html', $vars);
    }

    /**
     * Модуль: "разделитель"
     *
     * HTML-код с разделителем элементов
     *
     * @param array $vars Параметры
     */
    public function divider(array $vars = []): ?string
    {
        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'divider.html', $vars);
    }

    /**
     * Лазерная резка
     *
     * HTML-код с калькулятором лазерной резки
     *
     * @param array $vars Параметры
     */
    public function laser(array $vars = []): ?string
    {
        // Генерация представления
        return $this->view->render(DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . 'calculators' . DIRECTORY_SEPARATOR . 'laser.html', $vars);
    }

    /**
     * Рассчёт
     *
     * Генерирует ответ в виде ['expenses' => 0, 'income' => 0, 'profit' => 0]
     *
     * @param array $vars Параметры
     *
     * @todo
     * 1. Отправлять данные в зависимости от разрешения (обычным пользователям только expenses)
     */
    public function calculate(array $vars = []): ?string
    {
        // Инициализация журнала ошибок
        $vars['errors'] = ['calculators' => []];

        // Инициализация калькуляторов из тела запроса (подразумевается, что там массивы с параметрами)
        $calculators = json_decode(file_get_contents('php://input'), true);

        // Инициализация переменных для буфера вывода
        $machines = $managers = $engineers = $operators = 0;

        foreach ($calculators as $i => $calculator) {
            // Перебор калькуляторов

            foreach (['type'] as $parameter) {
                // Перебор мета-параметров

                // Инициализация общего параметра
                extract([$parameter => $calculator[$parameter] ?? null]);

                // Инициализация параметра для обработчика калькулятора
                unset($calculator[$parameter]);
            }

            // Инициализация номера калькулятора в его категории
            $number = count($vars['errors']['calculators'][$type] ?? []);

            // Инициализация журнала ошибок для калькулятора
            $calculator['errors'] = [];

            // Инициализация журнала ошибок для буфера вывода
            $vars['errors']['calculators'][$type][$number] = &$calculator['errors'];

            // Инициализация буфера параметров
            $parameters = [];

            // Инициализация параметра типа покупателя (подразумевается, что если не "entity", то "individual")
            $parameters['company'] = $calculator['buyer'] === 'entity';
            unset($calculator['buyer']);

            // Перенос остальных параметров в буфер параметров
            $parameters += $calculator;

            // var_dump($parameters);

            // Расчёт
            [$machines, $managers, $engineers, $operators] = calculators::$type(...$parameters);
        }

        return json_encode([
            'machines' => $machines,
            'managers' => $managers,
            'engineers' => $engineers,
            'operators' => $operators
        ]);
    }
}
